////////////////////////////////////////////////////////////////////////////////
//3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
//      10        20        30        40        50        60        70        80
//
// gl-particles
//
// events.c - implements all the signal callbacks
//
// Authors/Copyright 2009:
//    Mirco "MacSlow" Müller <macslow@bangang.de>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranties of
// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.
//
////////////////////////////////////////////////////////////////////////////////

#include <GL/gl.h>
#include <gdk/gdkkeysyms.h>

#include "events.h"
#include "opengl.h"

gboolean g_lmb_pressed = FALSE;
gdouble  g_last_pos[2] = {0.0f, 0.0f};

void
on_screen_changed (GtkWidget* widget,
		   GdkScreen* old_screen,
		   gpointer   data)
{
	GdkScreen*   screen   = gtk_widget_get_screen (widget);
	GdkColormap* colormap = gdk_screen_get_rgba_colormap (screen);

	if (!colormap)
	{
		g_print ("Could not get a RGBA-colormap. Using plain RGB.\n");
		colormap = gdk_screen_get_rgb_colormap (screen);
	}

	gtk_widget_set_colormap (widget, colormap);
}

void
on_window_realize (GtkWidget* widget,
		   gpointer   data)
{
	GdkGLContext*  gl_context  = gtk_widget_get_gl_context (widget);
	GdkGLDrawable* gl_drawable = gtk_widget_get_gl_drawable (widget);
	GLsizei        width       = widget->allocation.width;
	GLsizei        height      = widget->allocation.height;

	// make GL-context "current"
	if (!gdk_gl_drawable_gl_begin (gl_drawable, gl_context))
		return;

	opengl_init (width, height);

	// end drawing to current GL-context
	gdk_gl_drawable_gl_end (gl_drawable);
}

gboolean
on_window_configure (GtkWidget*         widget,
		     GdkEventConfigure* event,
		     gpointer           data)
{
	GdkGLContext*  gl_context  = gtk_widget_get_gl_context (widget);
	GdkGLDrawable* gl_drawable = gtk_widget_get_gl_drawable (widget);
	GLsizei        width       = widget->allocation.width;
	GLsizei        height      = widget->allocation.height;

	// make GL-context "current"
	if (!gdk_gl_drawable_gl_begin (gl_drawable, gl_context))
		return FALSE;

	opengl_resize (width, height);

	// end drawing to current GL-context
	gdk_gl_drawable_gl_end (gl_drawable);

	return FALSE;
}

gboolean
on_window_expose (GtkWidget*      widget,
		  GdkEventExpose* event)
{
	GdkGLContext*  gl_context  = gtk_widget_get_gl_context (widget);
	GdkGLDrawable* gl_drawable = gtk_widget_get_gl_drawable (widget);
	GLsizei        width       = widget->allocation.width;
	GLsizei        height      = widget->allocation.height;

	// make GL-context "current"
	if (!gdk_gl_drawable_gl_begin (gl_drawable, gl_context))
		return FALSE;

	opengl_draw (widget, width, height);

	if (gdk_gl_drawable_is_double_buffered (gl_drawable))
		gdk_gl_drawable_swap_buffers (gl_drawable);
	else
		glFlush ();

	// end drawing to current GL-context
	gdk_gl_drawable_gl_end (gl_drawable);

	return TRUE;
}

gboolean
on_delete (GtkWidget* widget,
	   GdkEvent*  event,
	   gpointer   data)
{
	opengl_cleanup ();
	gtk_main_quit ();

	return FALSE;
}

gboolean
draw_handler (GtkWidget* widget)
{
	gtk_widget_queue_draw (widget);

	return TRUE;
}

gboolean
pointer_handler (GtkWidget* widget)
{
	gdouble pos_delta[2] = {0.0f, 0.0f};
	gdouble damping      = 0.1f;
	gint    x            = 0;
	gint    y            = 0;

	// get pointer position and store it
	gtk_widget_get_pointer (widget, &x, &y);
	x += widget->allocation.width / 2;
	y += widget->allocation.height / 2;
	pos_delta[X] = g_last_pos[X] - (gdouble) x;
	pos_delta[Y] = g_last_pos[Y] - (gdouble) y;

	// don't do anything if LMB isn't pressed
	if (!g_lmb_pressed)
		return TRUE;

	if (pos_delta[X] != 0.0f)
	{
		opengl_camera_yaw (damping * pos_delta[X]);
	}

	if (pos_delta[Y] != 0.0f)
	{
		opengl_camera_pitch (damping * pos_delta[Y]);
	}

	g_last_pos[Y] = (gdouble) y;
	g_last_pos[X] = (gdouble) x;

	return TRUE;
}

gboolean
on_key_press (GtkWidget*   widget,
	      GdkEventKey* event,
	      gpointer     data)
{
	switch (event->keyval)
	{
		case GDK_Escape:
		case GDK_q:
		case GDK_Q:
			gtk_main_quit ();
		break;

		case GDK_space:
			opengl_camera_reset ();
		break;

		case GDK_w:
			opengl_camera_move_forward ();
		break;

		case GDK_s:
			opengl_camera_move_backward ();
		break;

		case GDK_d:
			opengl_camera_move_right ();
		break;

		case GDK_a:
			opengl_camera_move_left ();
		break;

		case GDK_r:
			opengl_camera_move_up ();
		break;

		case GDK_f:
			opengl_camera_move_down ();
		break;

		default :
		break;
	}

	return FALSE;
}

gboolean
on_button (GtkWidget*      widget,
	   GdkEventButton* event,
	   gpointer        data)
{
	gint x;
	gint y;

	if (event->type == GDK_BUTTON_PRESS &&
	    event->button == 1)
	{
		// get pointer position and store it
		gtk_widget_get_pointer (widget, &x, &y);
		x += widget->allocation.width / 2;
		y += widget->allocation.height / 2;
		g_last_pos[X] = (gdouble) x;
		g_last_pos[Y] = (gdouble) y;
		g_lmb_pressed = TRUE;
	}

	if (event->type == GDK_BUTTON_RELEASE &&
	    event->button == 1)
		g_lmb_pressed = FALSE;

	return FALSE;
}

